function design = DesignControllerAndObserverThermoFluid(p_sys)
% Design the centralized state observer and the
% state-feedback controller.
%
% Input:
%   p_sys       system model parameters
%
% Output:
%   design      struct including controller and estimator design
%
% --
% Supplementary material (simulation examples) to:
%   Distributed Event-based State Estimation (technical report)
%   Event-based State Estimation: An Emulation-based Approach (journal)
%
% Copyright 2015-17 Max Planck Society. All rights reserved.
% 
% Sebastian Trimpe
% Max Planck Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [20.04.15, ST]    first version
% [01.07.15, ST]    Implemented option to use (decentralized) controller
%                   from Sigurani et al. -> flag_whichController
%                   => activate controller by Sigurani
% [01.07.15, ST]    Activate LQR design, new tuning ("Design 2")
% [03.08.15, ST]    some clean up


%% Configuration
flag_mute = true;

% Which controller?
%   1: dlqr (used for papers)
%   2: controller from Sigurani et al. (used for development)
flag_whichController = 1;


%% Central State-feedback controller: LQR design

% Which design?
if flag_whichController==1
    % DLQR design
    
    % Weight matrices
%     % Design 1:
%     Q = diag([1 1 1 1]);
%     %R = 1e-2*diag([1 1e3 1 1e2]);
%     %R = 1e-2*diag([1 1e2 1 1e2]);
%     R = 1e-2*diag([1 1e1 1 1e1]);
%     N = [];
    
    % Design 2:
    Q = diag([1 0.01 1 0.01]);
    R = 1e-2*diag([1 1e1 1 1e1]);
    N = [];
    
    % Discrete time LQR design.
    [F,S,e] = dlqr(p_sys.A,p_sys.B,Q,R,N);
    
elseif flag_whichController==2
    % decentralized controllers in Sigurani et al
    K1 = [   10.5    0
        0.9     -0.05];
    K2 = [   11.5    0
        1.1     0.4 ];
    F = blkdiag(K1,K2);
    
end;

% Controller gain:
F = -1*F;     % positive feedback


% Some analysis
if ~flag_mute
    disp('State-feedback design:')
    disp(' ');
    disp('-F:');
    disp(-F);
    disp('Closed-loop eigenvalues:');
    disp(eig( p_sys.A+p_sys.B*F ));
end;

% Result
design.closedLoopDynCtrl = p_sys.A+p_sys.B*F;
design.F = F;


%% Central Luenberger Observer
% Model used for observer design
A_o = p_sys.est.A;
C_o = p_sys.est.C;

% Pick gain directly
L = 0.5*[   0.2 0 0 0;  % factor: 0.2; 0.5
        0 0.1 0 0; 
        0 0 0.2 0; 
        0 0 0 0.1 ];

% Some analysis
if ~flag_mute
    disp('Observer design:')
    disp(' ');
    disp('L:');
    disp(L);
    disp('Closed-loop eigenvalues:');
    disp(eig( (eye(length(A_o))-L*C_o)*A_o ));
end;

% Save results
design.L = L; 
design.closedLoopDynEst = (eye(length(A_o))-L*C_o)*A_o;


%% Event-based communication of measurements
% Transmit thresholds, one for each sensor.  EB_y.delta should have the 
% same structure as p_sys.y_dims.
delta_level = 0.01;     % tank level, []=m; Sigurani et al: 0.02
delta_temp = 0.2;       % temperature, []=K; Sigurani et al: 0.4
design.EB_y.delta{1} = {delta_level, delta_temp};
design.EB_y.delta{2} = {delta_level, delta_temp};

                
%% Synchronous averaging of estimates
design.syncAvg.K = inf;     % never


%% Event-based communication of inputs
% Same convention as for measurements.
delta_u = 0.02;
design.EB_u.delta{1} = {delta_u};
design.EB_u.delta{2} = {delta_u};
